/////////////////////////////////////////////////////////////////////////////
// Name:        editorcontainer.cpp
// Purpose:     Contains an editor, and can resize its control for
//              comfortable editing width. Also manages toolbars and
//              a pane at the bottom to be used for e.g. search.
// Author:      Julian Smart
// Modified by:
// Created:     2006-11-27
// RCS-ID:      $Id: editorcontainer.cpp,v 1.8 2007/06/18 04:57:35 anthemion Exp $
// Copyright:   (c) Julian Smart and Anthemion Software
// Licence:
/////////////////////////////////////////////////////////////////////////////

#include "wx/wx.h"

#include "wx/effects.h"

#include "editorcontainer.h"

/*
 * wxEditorContainerWindow implementation
 */

IMPLEMENT_CLASS(wxEditorContainerWindow, wxPanel)

BEGIN_EVENT_TABLE(wxEditorContainerWindow, wxPanel)
    EVT_PAINT(wxEditorContainerWindow::OnPaint)
    EVT_SIZE(wxEditorContainerWindow::OnSize)
    EVT_ERASE_BACKGROUND(wxEditorContainerWindow::OnEraseBackground)
END_EVENT_TABLE()

wxEditorContainerWindow::wxEditorContainerWindow(wxWindow *parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, int extraStyle)
{
    Init();
    Create(parent, id, pos, size, style, extraStyle);
}

bool wxEditorContainerWindow::Create(wxWindow *parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, int extraStyle)
{
    m_containerExtraStyle = extraStyle;

    if ((m_containerExtraStyle & wxEDITOR_EXTRA_STYLE_ALWAYS_FULL) == 0)
        style |= wxFULL_REPAINT_ON_RESIZE;

    wxPanel::Create(parent, id, pos, size, style);
    
    return true;
}

wxEditorContainerWindow::~wxEditorContainerWindow()
{
}

void wxEditorContainerWindow::Init()
{
    m_containerExtraStyle = 0;
    m_colour = *wxWHITE;
    m_useColour = true;
    m_contentWindow = NULL;
    m_lowerWindow = NULL;
    m_fullSize = false;
    m_useBestSize = false;

    m_minMarginLeftRight = wxEDITOR_CONTAINER_DEFAULT_LEFT_RIGHT_MARGIN;
    m_minMarginTopBottom = wxEDITOR_CONTAINER_DEFAULT_TOP_BOTTOM_MARGIN;
    m_maxContentWidth = wxEDITOR_CONTAINER_DEFAULT_TOP_MAX_CONTENT_WIDTH;
}

// Define the repainting behaviour
void wxEditorContainerWindow::OnPaint(wxPaintEvent& WXUNUSED(event))
{
    wxPaintDC dc(this);

    if (m_fullSize)
        return;

    if (GetUseColour() || !GetBackgroundBitmap().Ok())
    {
        dc.SetPen(wxPen(GetColour()));
        dc.SetBrush(wxBrush(GetColour()));
        dc.DrawRectangle(GetClientRect());
    }
    else
    {
        wxRect rect(GetClientRect());
        wxEffects effects;
#if wxCHECK_VERSION(2,8,0)
        effects.TileBitmap(rect, dc, GetBackgroundBitmap());
#else
        wxBitmap bitmap(GetBackgroundBitmap());
        effects.TileBitmap(rect, dc, bitmap);
#endif
        wxRect contentRect(m_currentContentRect);
        contentRect.x -= 1;
        contentRect.y -= 1;
        contentRect.width += 2;
        contentRect.height += 2;

        if (GetWindowStyleFlag() & wxEDITOR_STYLE_THIN_BORDER)
        {
            wxPen pen(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT));
            dc.SetPen(pen);
            dc.SetBrush(*wxTRANSPARENT_BRUSH);

            dc.DrawRectangle(contentRect);
        }
        else if (GetWindowStyleFlag() & wxEDITOR_STYLE_SUNKEN_BORDER)
        {
            effects.DrawSunkenEdge(dc, contentRect);
        }
    }
}

void wxEditorContainerWindow::OnEraseBackground(wxEraseEvent& event)
{
}

void wxEditorContainerWindow::OnSize(wxSizeEvent& event)
{
    PositionWindows();
}

void wxEditorContainerWindow::PositionWindows()
{
    wxRect clientRect = GetClientRect();

    //    printf("There are %d toolbars.\n", (int) m_toolbars.GetCount());
    
    // Position toolbars
    wxNode* node = m_toolbars.GetFirst();
    while (node)
    {
        wxWindow* toolbar = (wxWindow*) node->GetData();
        if (toolbar->IsShown())
        {
            wxSize sz = toolbar->GetSize();
            toolbar->SetSize(0, clientRect.y, clientRect.width, sz.y);
            clientRect.y += sz.y;
            clientRect.height -= sz.y;
        }
        
        node = node->GetNext();
    }
    
    // Deal with the lower window
    if (IsLowerWindowShown())
    {
        wxSize sz = GetLowerWindow()->GetSize();
        if (sz.y > clientRect.height)
            sz.y = clientRect.height;
        
        GetLowerWindow()->SetSize(0, clientRect.y, clientRect.width, sz.y);
        clientRect.height -= sz.y;
    }
    
    // Finally position the content window
    if (GetContentWindow())
    {
        int contentWidth;
        int contentHeight;
        int contentX;
        int contentY;
        if (m_fullSize || (clientRect.width <= m_maxContentWidth))
        {
            int availableWidth = clientRect.width;
            int availableHeight = wxMax(1, clientRect.height);
            contentWidth = availableWidth;
            contentHeight = availableHeight;
            contentX = 0;
            contentY = clientRect.y;
        }
        else if (m_useBestSize)
        {
            wxSize bestSize = GetContentWindow()->GetBestSize();
            contentWidth = bestSize.x;
            contentHeight = bestSize.y;
            contentX = (int)((clientRect.width - contentWidth)/2);
            contentY = clientRect.y + (int)((clientRect.height - contentHeight)/2);            
        }
        else
        {
            int availableWidth = clientRect.width - m_minMarginLeftRight*2;
            int availableHeight = wxMax(1, (clientRect.height - m_minMarginTopBottom*2));
            contentWidth = wxMin(availableWidth, m_maxContentWidth);
            contentHeight = availableHeight;
            contentX = (int)((clientRect.width - contentWidth)/2);
            contentY = clientRect.y + (int)((clientRect.height - contentHeight)/2);            
        }

        m_currentContentRect = wxRect(contentX, contentY, contentWidth, contentHeight);

        GetContentWindow()->SetSize(contentX, contentY, contentWidth, contentHeight);
    }
}

/// Show/hide lower window
void wxEditorContainerWindow::ShowLowerWindow(bool show)
{
    if (GetContentWindow())
        GetContentWindow()->Show(show);
    PositionWindows();
}

/// Add a toolbar to be managed by the container (could be any window,
// but in toolbar position)
bool wxEditorContainerWindow::AddToolBar(wxWindow* toolbar, bool layout)
{
    if (!m_toolbars.Member(toolbar))
        m_toolbars.Append(toolbar);

    if (layout)
        PositionWindows();
    
    return true;
}

/// Remove a toolbar
bool wxEditorContainerWindow::RemoveToolBar(wxWindow* toolbar, bool deleteWindow, bool layout)
{
    wxNode* node = m_toolbars.Find(toolbar);
    if (node)
    {
        wxWindow* win = (wxWindow*) node->GetData();
        delete node;
        if (deleteWindow)
            win->Destroy();
        return true;
    }
    return false;
}

/// Has this toolbar
bool wxEditorContainerWindow::HasToolBar(wxWindow* toolbar) const
{
    return (m_toolbars.Find(toolbar) != NULL);
}

/// Clear toolbars
bool wxEditorContainerWindow::ClearToolBars(bool deleteWindows)
{
    // Position toolbars
    wxNode* node = m_toolbars.GetFirst();
    while (node)
    {
        wxNode* next = node->GetNext();
        wxWindow* toolbar = (wxWindow*) node->GetData();
        delete node;
        if (deleteWindows)
            toolbar->Destroy();
        
        node = next;
    }
    return true;
}

/// Get toolbar
wxWindow* wxEditorContainerWindow::GetToolBar(int n) const
{
    int count = GetToolBarCount();
    wxASSERT (n < count && n > -1);
    if (n < count && n > -1)
    {
        wxNode* node = m_toolbars.Item(n);
        return (wxWindow*) node;        
    }
    return NULL;
}

