/////////////////////////////////////////////////////////////////////////////
// Name:        editorcontainer.h
// Purpose:     Contains an editor, and can resize its control for
//              comfortable editing width. Also manages toolbars and
//              a pane at the bottom to be used for e.g. search.
// Author:      Julian Smart
// Modified by:
// Created:     2006-11-27
// RCS-ID:      $Id: editorcontainer.h,v 1.6 2007/06/16 23:42:17 anthemion Exp $
// Copyright:   (c) Julian Smart and Anthemion Software
// Licence:
/////////////////////////////////////////////////////////////////////////////

#ifndef _ANTHEMION_EDITORCONTAINER_H_
#define _ANTHEMION_EDITORCONTAINER_H_

#include "wx/panel.h"

#define wxEDITOR_CONTAINER_DEFAULT_LEFT_RIGHT_MARGIN        5
#define wxEDITOR_CONTAINER_DEFAULT_TOP_BOTTOM_MARGIN        5
#define wxEDITOR_CONTAINER_DEFAULT_TOP_MAX_CONTENT_WIDTH    400

// Border styles when not maximized within the container
#define wxEDITOR_STYLE_THIN_BORDER   0x00001000
#define wxEDITOR_STYLE_SUNKEN_BORDER 0x00002000

// Extra styles

// Can optimize refreshing if we know we're always going to be full size
#define wxEDITOR_EXTRA_STYLE_ALWAYS_FULL        0x01

/*!
 * The editor container
 */

class wxEditorContainerWindow: public wxPanel
{
 public:
    /// Ctors
    wxEditorContainerWindow() { Init(); }
    wxEditorContainerWindow(wxWindow *parent, wxWindowID id = wxID_ANY,
        const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = 0, int extraStyle = 0);
    ~wxEditorContainerWindow();

    bool Create(wxWindow *parent, wxWindowID id = wxID_ANY,
        const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = 0, int extraStyle = 0);

    void Init();

//// Event handlers
    
    void OnPaint(wxPaintEvent& event);
    void OnEraseBackground(wxEraseEvent& event);
    void OnSize(wxSizeEvent& event);

//// Accessors

    /// Set/get content window.
    void SetContentWindow(wxWindow* win) { m_contentWindow = win; }
    wxWindow* GetContentWindow() const { return m_contentWindow; }

    /// Set get lower window
    void SetLowerWindow(wxWindow* win) { m_lowerWindow = win; }
    wxWindow* GetLowerWindow() const { return m_lowerWindow; }

    /// Set/get background bitmap
    void SetBackgroundBitmap(const wxBitmap& bitmap) { m_backgroundBitmap = bitmap; }
    const wxBitmap& GetBackgroundBitmap() const { return m_backgroundBitmap; }

    /// Set/get background colour
    const wxColour& GetColour() const { return m_colour; }
    void SetColour(const wxColour& colour) { m_colour = colour; }

    /// Set/get background painting mode
    bool GetUseColour() const { return m_useColour; }
    void SetUseColour(bool useColour) { m_useColour = useColour; }

    /// Set/get full-size mode, i.e. eliminate background drawing because we
    /// know we won't be showing any window
    bool GetFullSize() const { return m_fullSize; }
    void SetFullSize(bool fullSize) { m_fullSize = fullSize; }

    /// Specify whether the best size of the content window should be used,
    /// in case it's e.g. a panel that should take a fixed size (assuming
    /// not using full size).
    bool GetUseBestSize() const { return m_useBestSize; }
    void SetUseBestSize(bool useBestSize) { m_useBestSize = useBestSize; }

    /// Set/get extra style
    int GetContainerExtraStyle() const { return m_containerExtraStyle; }
    void SetContainerExtraStyle(int extraStyle) { m_containerExtraStyle = extraStyle; }

    /// Set/get minimum left/right margins for editor
    int GetLeftRightMargin() const { return m_minMarginLeftRight; }
    void SetLeftRightMargin(int margin) { m_minMarginLeftRight = margin; }

    /// Set/get minimum top-bottom margins for editor
    int GetTopBottomMargin() const { return m_minMarginTopBottom; }
    void SetTopBottomMargin(int margin) { m_minMarginTopBottom = margin; }

    /// Set/get maximum width of the content window
    int GetMaxContentWidth() const { return m_maxContentWidth; }
    void SetMaxContentWidth(int w) { m_maxContentWidth = w; }

/// Content and lower window operations

    /// Show/hide lower window
    void ShowLowerWindow(bool show);

    /// Is lower window shown?
    bool IsLowerWindowShown() const { return m_lowerWindow && m_lowerWindow->IsShown(); }

///  Toolbar operations

    /// Add a toolbar to be managed by the container (could be any window,
    // but in toolbar position)
    bool AddToolBar(wxWindow* toolBar, bool layout = false);

    /// Remove a toolbar
    bool RemoveToolBar(wxWindow* toolBar, bool deleteWindow = true, bool layout = false);

    /// Has this toolbar
    bool HasToolBar(wxWindow* toolBar) const;

    /// Clear toolbars
    bool ClearToolBars(bool deleteWindows = true);

    /// Get toolbar count
    int GetToolBarCount() const { return m_toolbars.GetCount(); }

    /// Get toolbar
    wxWindow* GetToolBar(int n) const;

/// Other operations
    
    /// Lay out
    void Update() { PositionWindows(); Refresh(); }

//// Implementation

    // Position subwindows
    void PositionWindows();

    DECLARE_CLASS(wxEditorContainerWindow)
    DECLARE_EVENT_TABLE()

protected:
    wxBitmap        m_backgroundBitmap;
    wxColour        m_colour;
    bool            m_useColour;
    wxWindow*       m_contentWindow;
    wxList          m_toolbars;
    wxWindow*       m_lowerWindow;

    int             m_minMarginLeftRight;
    int             m_minMarginTopBottom;
    int             m_maxContentWidth;

    wxRect          m_currentContentRect;

    bool            m_fullSize;
    bool            m_useBestSize;

    int             m_containerExtraStyle;
};

#endif
    // _ANTHEMION_EDITORCONTAINER_H_

